#!/usr/bin/perl -w
# It Makes HTML file index from the current directory including sub-directories
# in the table named 'htmlindex' of the db named 'pisd'
# in the local MySQL server.  DB Search HTML files.
#
# Copyright by gang(at)users.kldp.net with the license of GNU General Public 
# License version 2
#
# Change $baseurl to an appropriate string.

use DBI;

$baseurl = "http://localhost/";
$follow_symlink = TRUE; # WARNING! If true, it does not check circular link,
						# which may cause infinite loop.

# Main Procedures: call functions
mysqlConnect();
search_dir(".");
deletion_check();
mysqlDisconnect();
# End of Main Procedures

# Followings are functions

sub inside_of_tag
{
    if( $_[0] =~ /<$_[1]>(.+)<\/$_[1]>/i)
    {
        my $ret = $1;
        $ret =~ s/\|/!/;
        return $ret;
    }
    else
    {
        return "";
    }
}

sub meta_content
{
    if( $_[0] =~ /<meta[^\>]*name\s*=\s*("?)$_[1]\1[^\>]*content\s*=\s*("?)([^"]*)\2[^\>]*>/i )
    {
        my $ret = $3;
        $ret =~ s/\|/!/;
        return $ret;
    }
    else
    {
        return "";
    }
}

sub search_descriptions
{
    open(HTML_FILE, $_[0]) || die "Erorr in opening $_[0]\n";

    my $html = "";
    while(<HTML_FILE>)
    {
        chomp;
        $html .= $_;
    }

    $title = inside_of_tag($html,"title");
    if(!$title)
    {
        $title = inside_of_tag($html, "h1");
    }
    if(!$title)
    {
        $title = inside_of_tag($html, "h2");
    }

    $author     = meta_content($html, "author");
    $keywords   = meta_content($html, "keywords");
    $keywords   =~ s/\s*,\s*/,/g;

    close(HTML_FILE) || die "Error in closing\n";
}

sub search_dir
{
    my($dir) = @_;

    opendir(DIR,$dir) or return;
    my @filelist = readdir(DIR);
    closedir(DIR);

    my $i;
    foreach $i (@filelist)
    {
        next if (($i eq ".") or ($i eq ".."));
        my $file = $dir."/".$i;
        $file =~ s/^\.\///;
        if((-d $file) and ($follow_symlink or !(-l $file)) )
        {
            search_dir($file);
        }
        elsif ( $file =~ /\.html?/i )
        {
            my ($mtime) = (stat($file))[9,10];
            #my ($mtime,$ctime) = (stat($file))[9,10];
            search_descriptions($file);
            if($title){
                $serviceId = $file;
                $accessMethod = $baseurl . $file ;
                ($sec,$min,$hour,$mday,$mon,$year) = gmtime($mtime);
                $lastModified = sprintf("%02d%02d%02d%02d%02d%02d",
                    ($year-100),($mon+1),($mday+1),$hour,$min,$sec);
                #print "$file|$title|$accessMethod|$mtime|$author|$keywords\n";
                updateIndex();
            }
        }
    }
}

sub mysqlConnect
{
     # Connect to the database.
     $dbh = DBI->connect("DBI:mysql:database=pisd;host=localhost",
                "gang", "",
                {'RaiseError' => 1});
}

sub updateIndex
{
     # Retrieve data from the table.
     my $query="SELECT serviceId, title, modifiedTime FROM htmlindex WHERE serviceId = '". $serviceId . "'";
     #print $query."\n";

     my $sth = $dbh->prepare($query);
     $sth->execute();

     my $ref = $sth->fetchrow_hashref();
     if($ref)
     {
	 	#print $ref;
        print "Found a row in MySQL DB: id = $ref->{'serviceId'}, modified = $ref->{'modifiedTime'}";
        $sth->finish();
        if( $ref->{'title'} and ($ref->{'modifiedTime'} ge $lastModified ))
        {
            print " - already updated.\n";
            return;
        }
		else 
		{
			#print "\n". $ref->{'modifiedTime'} ."\n". $lastModified . "\n";
		}
     }
     else
     {
        #print "not found\n";
     }
     
     $query = "REPLACE INTO htmlindex VALUES (".$dbh->quote($serviceId).", ".
            $dbh->quote($title). ", ".
            $dbh->quote($accessMethod). ", ".
            $dbh->quote($author). ", ".
            $dbh->quote($keywords). ",NULL)";
     print "\n".$query."\n";
     $dbh->do($query);

     # Same thing, but using placeholders
     #$dbh->do("INSERT INTO foo VALUES (?, ?)", undef, 2, "Jochen");
}

sub deletion_check 
{
     # Retrieve data from the table.
     my $query="SELECT serviceId FROM htmlindex WHERE title IS NOT NULL";
     #print $query."\n";

     my $sth = $dbh->prepare($query);
     $sth->execute();

	 my $ref = 0;
     while($ref = $sth->fetchrow_hashref())
	 {
		if(!(-e ($ref->{'serviceId'})))
		{
        	print "Found a deleted serviceId : ".$ref->{'serviceId'}."\n";
			$query = "UPDATE htmlindex SET title = NULL WHERE serviceId = '".$ref->{'serviceId'} ."'";
			print $query."\n";
			my $sth2 = $dbh->prepare($query);
			$sth2->execute();
		}
		else
		{
			print "Valid service: ". $ref->{'serviceId'}."\n";
		}
	 }
}
		
sub mysqlDisconnect
{
     # Disconnect from the database.
     $dbh->disconnect();
}
