#include <iostream.h>
#include "pdu.h"

/* Makes the binary string in the static buffer from a PISD message.
 * The returnend value might be obsolted when another call
 * of this function happens.
 * If any error, returns NULL.
 */
char *getPdu(PisdMessage &msg, int &len)
{
	static char buf[PDU_BUFSIZE];
	AsnBuf asnBuf;
	AsnLen asnLen(len);

	asnBuf.Init(buf,PDU_BUFSIZE);
	asnBuf.ResetInWriteRvsMode();
	if( msg.BEncPdu(asnBuf, asnLen) == false )
	{
		return NULL;
	}
	else
	{
		len = (int)asnLen;
		return asnBuf.DataPtr();
	}
}

/* Makes the PISD message in static area from binary string.
 * The returnend value might be obsolted when another call
 * of this function happens.
 * If any error, returns NULL.
 */
PisdMessage *decodePdu(char *pdu)
{
	int len = pduLength(pdu);
	static PisdMessage msg;
	AsnBuf asnBuf;
	AsnLen l(len);
	asnBuf.ResetInWriteRvsMode();
	asnBuf.InstallData((char *)pdu,l);
	if( msg.BDecPdu(asnBuf,l) == false ) return NULL;
	return &msg;
}

/* Returns the length of BER encoded PDU */
unsigned long long pduLength(const char *pdu)
{
	unsigned long long dataLen = 0;
	unsigned headerLen = 2;
	const char *ptr = pdu + 1;

	// skip tag field
	if( (pdu[0] & 0x1F) == 0x1F )
	{
		while((*ptr)&0x80)
		{
			ptr++;
			headerLen++;
		}
		ptr++;
		headerLen++;
	}

	if( *ptr & 0x80 )
	{
		int byteLen = *ptr & 0x7F;
		ptr++;
		if(byteLen)
		{
			for(int i=0; i<byteLen; i++)
			{
				dataLen = (dataLen<<8) + *ptr++;
				headerLen++;
			}
		}
		else
		{
			headerLen += 2;
			while(1)
			{
				if( (*ptr==0) && (*(ptr+1)==0) ) break;
				dataLen = dataLen<<8 + *ptr++;
				headerLen++;
			}
		}
	}
	else
	{
		dataLen = *ptr;
	}

	return headerLen + dataLen;
}

/* Print out hex code of BER encoded PDU to stdout */
void printPdu(const char *pdu)
{
	unsigned long long len = pduLength(pdu);

	for(unsigned long long i=0; i<len; i++)
	{
		cout << (int)(unsigned char)pdu[i] << ' ';
	}
	cout << endl;
}
