#include <sys/socket.h>
#include <netinet/in.h> // sockaddr_in, htonl
#include <arpa/inet.h> // inet_pton
#include <string.h>
#include <errno.h>
#include "pisd_net.h"
#include "pdu.h"

/* TCP connection from client */
int connectToServer(const char *server)
{
	int sock;
	struct sockaddr_in servaddr;

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if(sock < 0) return -1;
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(PISD_SERVER_PORT);
	inet_pton(AF_INET, server, &servaddr.sin_addr);
	if( connect(sock, (const struct sockaddr *)&servaddr,
				sizeof(servaddr)) < 0)
	{
		close(sock);
		return -1;
	}

	return sock;
}

/* TCP server */
int startServer()
{
	int sock;
	struct sockaddr_in cliaddr, servaddr;

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if(sock < 0) return -1;

	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port = htons(PISD_SERVER_PORT);

	if(bind(sock, (const struct sockaddr *)&servaddr, 
				sizeof(servaddr)) < 0)
	{
		close(sock);
		return -1;
	}

	if(listen(sock, 5) < 0)
	{
		close(sock);
		return -1;
	}

	return sock;
}

/* TCP read upto n bytes*/
ssize_t readn(int fd, void *vptr, size_t n)
{
	size_t nleft = n;
	ssize_t nread;
	char *ptr = (char *)vptr;

	while(nleft > 0)
	{
		if( (nread = read(fd, ptr, nleft)) < 0 )
		{
			if(errno == EINTR)
			{
				nread = 0;
			}
			else
			{
				return -1;
			}
		}
		else
		{
			if(nread == 0) break;
		}

		nleft -= nread;
		ptr += nread;
	}
	return (n-nleft);
}

/* TCP write n bytes*/
ssize_t writen(int fd, void *vptr, size_t n)
{
	size_t nleft = n;
	ssize_t nwritten;
	const char *ptr = (const char *)vptr;

	while(nleft > 0)
	{
		if( (nwritten = write(fd, ptr, nleft)) <= 0)
		{
			if(errno == EINTR)
			{
				nwritten = 0;
			}
			else
			{
				return -1;
			}
		}

		nleft -= nwritten;
		ptr += nwritten;
	}
	return n;
}

/* Allocates new memory for a PDU, and read it in TCP */
char *readPdu(int sock)
{
	char tmpBuf[6];
	int n = readn(sock, tmpBuf, 6);
	unsigned long long len = pduLength(tmpBuf);
	char *pdu = new char[len];

	if(n<0)
	{
		return NULL;
	}

	memcpy(pdu, tmpBuf, n);
	if(len > n)
	{
		if(readn(sock, pdu + n, len - n) < 0)
		{
			delete[] pdu;
			return NULL;
		}
	}

	return pdu;
}
