/* Filename: r_update_start.C
 * Realm Clinet that starts with an UpdateStart message.
 */

#include <iostream.h>
#include <string.h>
#include <sys/socket.h>
#include "realm_htmlindex.h"
#include "pdu.h"
#include "pisd_net.h"

char *latestTimeOfHtmlindex();

int main(int argc, char **argv)
{
	if( argc != 2 )
	{
		cerr << "Usage: r_update_start <PISDS>\n";
		return 0;
	}
	const char *pisds = argv[1];

	char *latestTime = latestTimeOfHtmlindex();
	if(!latestTime)
	{
		cerr << "No services\n";
		return 0;
	}
	//cout<< "The latest service is modified at " << latestTime << ".\n";

	// connect to PISDS
	int sock = connectToServer(pisds);
	if(sock < 0)
	{
		cerr << "Error: cannot connect to server '" << pisds << "'.\n";
		return 0;
	}

	// send update start; realm: htmlindex, host: localhost
	int len;
	RealmUpdateStart rus("htmlindex", "localhost", latestTime);
	PisdMessage *msg = rus.toPisdMessage();
	char *pdu = getPdu(*msg, len);
	//cout << *msg << endl;
	delete msg;

	if(writen(sock, pdu, len) < 0)
	{
		cerr << "Error: cannot send update start.\n";
		close(sock); 
		return 0; 
	}
	//printPdu(pdu);

	// get difference requeset
	pdu = readPdu(sock);
	if(!pdu)
	{
		cerr << "Error: cannot receive difference request.\n";
		close(sock); 
		return 0; 
	}
	msg = decodePdu(pdu);
	delete pdu;
	if(!msg)
	{
		cerr << "Error: cannot decode pdu.\n";
		close(sock);
		return 0;
	}
	if(msg->choiceId != PisdMessage::differenceRequestCid)
	{
		cerr << "Error: difference request is not received.\n";
		close(sock);
		return 0;
	}
	cout << "Received Difference Request:\n" << *msg << endl;
	RealmDifferenceRequest rdr(*msg->differenceRequest);
	cout << rdr.since << rdr.realm << rdr.maxLogs << endl;

	// make index
	HtmlindexDifferenceResponse hdr(rdr.since, rdr.maxLogs);
	if(!hdr.isValidResponse())
	{
		cerr << "No vaild resoponse.\n";
		close(sock);
		return 0;
	}
	do
	{
		// difference response
		msg = hdr.toPisdMessage();
		pdu = getPdu(*msg, len);
		delete msg;
		if(writen(sock, pdu, len) < 0)
		{
			cerr << "Error: cannot send difference response.\n";
			close(sock);
			return 0;
		}

		// get status notify
		pdu = readPdu(sock);
		if(!pdu)
		{
			cerr << "Error: cannot receive status notify.\n";
			close(sock);
			break;
		}
		msg = decodePdu(pdu);
		delete pdu;
		if(!msg)
		{
			cerr << "Error: cannot decode pdu.\n";
			close(sock);
			break;
		}
		if(msg->choiceId != PisdMessage::statusNotifyCid)
		{
			cerr << "Error: status notify is not received.\n";
			close(sock);
			break;
		}
		cout << "Received Status Notify:\n" << *msg << endl;

	} while(hdr.isHaveMore() && (msg->differenceResponse->haveMore != NULL)
				&& (bool)*(msg->differenceResponse->haveMore));

	close(sock);
	return 0;
}

/* Gets the latest modifiedTime from htmlindex table. */
char *latestTimeOfHtmlindex()
{
	static char modifiedTime[30];
	MYSQL mysql;

	mysql_init(&mysql);
	if (!mysql_real_connect(&mysql, MYSQL_CLIENT_HOST, MYSQL_CLIENT_USER,
				MYSQL_CLIENT_PASSWD, MYSQL_CLIENT_DB, 0, NULL, 0))
	{
		cerr<< "Failed to connect to database: Error: "
			<< mysql_error(&mysql)
			<< endl;
		return NULL;
	}

	const char *queryStr = "SELECT modifiedTime FROM " MYSQL_CLIENT_TABLE \
		" ORDER BY modifiedTime DESC LIMIT 1";
	if(mysql_query(&mysql, queryStr))
	{
		cerr<< "Failed to query: Error: "
			<< mysql_error(&mysql)
			<< " - " 
			<< queryStr
			<< endl;
		mysql_close(&mysql);
		return NULL;
	}
	MYSQL_RES *mysqlResult= mysql_store_result(&mysql);
	if(!mysqlResult)
	{
		cerr<< "Failed to get result: Error: "
			<< mysql_error(&mysql)
			<< endl;
		mysql_close(&mysql);
		return NULL;
	}
	MYSQL_ROW row = mysql_fetch_row(mysqlResult);
	if(!row)
	{
		cerr<< "Error: cannot fetch a row\n";
		mysql_close(&mysql);
		return NULL;
	}

	strcpy(modifiedTime, row[0]);
	strcat(modifiedTime, "Z");
	mysql_close(&mysql);

	return modifiedTime;
}
