#include <string.h>
#include "realm.h"

// make duplicated string
static char *strDup(const char *str,int len=0)
{
	if(!str) return NULL;
	if(!len) len = strlen(str);
	char *ret = new char[len+1];
	for(int i=0; i<len; i++) ret[i] = str[i];
	ret[len] = '\0';
	return ret;
}

/* class RealmLog */

void RealmLog::set(const char *s, const char *m, const bool v, 
		const char *t, const char *a)
{
	serviceId = strDup(s);
	modifiedTime = strDup(m);
	valid = v;
	if(v)
	{
		title = strDup(t);
		accessMethod = strDup(a);
	}
	else
	{
		title = accessMethod = NULL;
	}
}

void RealmLog::set(const ServiceLog &sl)
{
	char *s, *m, *t, *a;
	bool v;

	s = utf8AsnOcts2LocalStr(sl.serviceId);
	m = utf8AsnOcts2LocalStr(sl.modifiedTime);
	v = (sl.description->choiceId == ServiceLogChoice::validServiceCid);
	if(v)
	{
		t = utf8AsnOcts2LocalStr((char *)sl.description->validService->title);
		a = utf8AsnOcts2LocalStr((char *)sl.description->validService->accessMethod);
	}
	else
	{
		t = a = NULL;
	}

	set(s, m, v, t, a);
}

void RealmLog::reset(const char *s, const char *m, const bool v, 
		const char *t, const char *a)
{	
	clear();
	set(s, m, v, t, a);
}

void RealmLog::reset(const ServiceLog &sl)
{
	clear();
	set(sl);
}

void RealmLog::clear()
{
	delete serviceId;
	delete modifiedTime;
	delete title;
	delete accessMethod;
}

RealmLog::RealmLog()
{
	serviceId = modifiedTime = title = accessMethod = NULL;
	valid = false;
}

ServiceLog *RealmLog::toServiceLog(ServiceLog &sl)
{
	if( (!serviceId) || (!modifiedTime) ) return NULL;

	sl.serviceId = localChar2Utf8Char(serviceId);
	sl.modifiedTime = localChar2Utf8Char(modifiedTime);
	sl.description = new ServiceLogChoice;
	if(valid)
	{
		sl.description->choiceId = ServiceLogChoice::validServiceCid;
		sl.description->validService = new ServiceDescription;
		sl.description->validService->title = localChar2Utf8Char(title);
		sl.description->validService->accessMethod
			= (IA5String)(localChar2Utf8Char(accessMethod));
	}
	else
	{
		sl.description->choiceId = ServiceLogChoice::invalidServiceCid;
		sl.description->invalidService= new AsnNull;
	}

	return &sl;
}

ServiceLog *RealmLog::toServiceLog()
{
	if( (!serviceId) || (!modifiedTime) ) return NULL;

	ServiceLog *sl = new ServiceLog();
	return toServiceLog(*sl);
}


/* Functions */

/* Makes difference response PDU from a list of logs */
PisdMessage *makeDifferenceResponsePisdMessage(RealmLog *array, int size,
		bool haveMore)
{
	if( (!array) || (!size) ) return NULL;
	
	PisdMessage *mesg = new PisdMessage;
	mesg->choiceId = PisdMessage::differenceResponseCid;
	mesg->differenceResponse = new DifferenceResponse();

	ServiceLog sl;

	for(int i=0; i<size; i++)
	{
		if(array[i].toServiceLog(sl))
		{
			mesg->differenceResponse->serviceLogs.AppendCopy(sl);
		}
	}
	mesg->differenceResponse->haveMore = new AsnBool(haveMore);

	return mesg;
}


/* class RealmUpdateStart */

RealmUpdateStart::RealmUpdateStart(const char *r, const char *h, const char *l)
{
	realm = strDup(r);
	host = strDup(h);
	latestTime = strDup(l);
}

RealmUpdateStart::RealmUpdateStart(UpdateStart &us)
{
	realm = utf8AsnOcts2LocalStr(us.realm);
	host = us.host ? utf8AsnOcts2LocalStr(*us.host) : host = NULL;
	latestTime = utf8AsnOcts2LocalStr(us.latestTime);
}

RealmUpdateStart::~RealmUpdateStart()
{
	delete realm;
	delete host;
	delete latestTime;
}

PisdMessage *RealmUpdateStart::toPisdMessage()
{
	PisdMessage *mesg = new PisdMessage();

	mesg->choiceId = PisdMessage::updateStartCid;
	mesg->updateStart = new UpdateStart();
	mesg->updateStart->realm = localChar2Utf8Char(realm);
	mesg->updateStart->host 
		= host ? new IA5String(localChar2Utf8Char(host)): NULL;
	mesg->updateStart->latestTime = localChar2Utf8Char(latestTime);

	return mesg;
}

/* class RealmDifferenceRequest */

RealmDifferenceRequest::RealmDifferenceRequest(const char *s, const char *r,
		const int m)
{
	since = strDup(s);
	realm = strDup(r);
	maxLogs = m;
}

RealmDifferenceRequest::RealmDifferenceRequest(DifferenceRequest &dr)
{
	since = utf8AsnOcts2LocalStr(dr.since);
	realm = dr.realm ? utf8AsnOcts2LocalStr(*dr.realm) : NULL;
	maxLogs = (dr.maxLogs != NULL) ? (int)*(dr.maxLogs) : 0;
}

RealmDifferenceRequest::~RealmDifferenceRequest()
{
	delete since;
	delete realm;
}

PisdMessage *RealmDifferenceRequest::toPisdMessage()
{
	PisdMessage *mesg = new PisdMessage();

	mesg->choiceId = PisdMessage::differenceRequestCid;
	mesg->differenceRequest = new DifferenceRequest();
	mesg->differenceRequest->since = localChar2Utf8Char(since);
	mesg->differenceRequest->realm 
		= realm ? new UTF8String(localChar2Utf8Char(realm)): NULL;
	mesg->differenceRequest->maxLogs = maxLogs ? new AsnInt(maxLogs) : NULL;

	return mesg;
}

/* class RealmStatusNotify */

RealmStatusNotify::RealmStatusNotify(const int c, const char *r)
{
	code = c;
	readableString = strDup(r);
}

RealmStatusNotify::RealmStatusNotify(StatusNotify &fr)
{
	code = fr.code;
	readableString = fr.readableString ? 
		utf8AsnOcts2LocalStr(*fr.readableString) : NULL;
}

RealmStatusNotify::~RealmStatusNotify()
{
	delete readableString;
}

PisdMessage *RealmStatusNotify::toPisdMessage()
{
	PisdMessage *mesg = new PisdMessage();

	mesg->choiceId = PisdMessage::statusNotifyCid;
	mesg->statusNotify = new StatusNotify();
	mesg->statusNotify->code = code;
	mesg->statusNotify->readableString = readableString ? 
		new UTF8String(localChar2Utf8Char(readableString)) : NULL;

	return mesg;
}

// UTF-8 <-> Local Char Set
char *localChar2Utf8Char(const char *localStr) /* no memory allocation; use static mem */
{
	/* do not convert any code now */
	return (char *)localStr;
}

char *utf8AsnOcts2LocalStr(const AsnOcts &asnOcts) /* new memory allocation */
{
	/* do not convert any code now */
	int len = asnOcts.Len();
	if(len == 0) return NULL;
	char *str = new char[len+1];
	const char *p = (const char *)asnOcts;
	for(int i=0; i<len; i++)
	{
		str[i] = p[i];
	}
	str[len] = '\0';
	return str;
}

