#ifndef REALM_H
#define REALM_H

#include <asn-incl.h>
#include "simple_pisd_asn1.h"

/* A service log with friend data members.
 * Has converters that converts itself from/to Asn types.
 */
class RealmLog{
	protected:
		/* member variables describing a service log */
		char *serviceId;
		char *modifiedTime;
		bool valid;
		char *title;
		char *accessMethod;

		/* member variable operations */
		// set member variables
		void set(const char *s, const char *m, const bool v,
				const char *t, const char *a);
		void set(const ServiceLog &sl);

		// set member variables after clearing old members
		void reset(const char *s, const char *m, const bool v,
				const char *t, const char *a);
		void reset(const ServiceLog &sl);

		// clear old member variables
		void clear();

	public:
		/* constructors */
		// set all members NULL
		RealmLog();
		RealmLog(const char *s, const char *m, const bool v,
				const char *t=NULL, const char *a=NULL)
			{ set(s, m, v, t, a); }
		RealmLog(const ServiceLog &sl)
			{ set(sl); }
		/* destructor */
		~RealmLog()
			{ clear(); }

		/* accessor */
		const char *getServiceId() { return serviceId; }
		const char *getModifiedTime() { return modifiedTime; }
		const char *getTitle() { return title; }
		const char *getAccessMethod() { return accessMethod; }
		bool isValid() { return valid; }

		/* convertor */
		// convert to ServiceLog; return -1 if fail.
		ServiceLog *toServiceLog();
		ServiceLog *toServiceLog(ServiceLog &sl);
		// convert from ServiceLog
		RealmLog &operator = (const ServiceLog &sl)
			{ reset(sl); return *this; }
};

/* make an pisd message (Asn type) form an service log array */
PisdMessage *makeDifferenceResponsePisdMessage(RealmLog *array, int size,
		bool haveMore);

/* UpdateStart PISD message */
class RealmUpdateStart
{
	protected:
	public:
		char *realm;
		char *host;
		char *latestTime;

	public:
		RealmUpdateStart(const char *r=NULL, const char *h=NULL,
				const char *l=NULL);
		RealmUpdateStart(UpdateStart &us);
		~RealmUpdateStart();
		PisdMessage *toPisdMessage();
};

/* DifferenceRequest PISD message */
class RealmDifferenceRequest
{
	public:
		char *since;
		char *realm;
		int maxLogs;

		RealmDifferenceRequest(const char *s=NULL, const char *r=NULL,
				const int m=0);
		RealmDifferenceRequest(DifferenceRequest &dr);
		~RealmDifferenceRequest();
		PisdMessage *toPisdMessage();
};

/* StatusNotify PISD message */
class RealmStatusNotify
{
	public:
		int code;
		char *readableString;

		RealmStatusNotify(const int c=0, const char *r=NULL);
		RealmStatusNotify(StatusNotify &fr);
		~RealmStatusNotify();
		PisdMessage *toPisdMessage();
};

/* converts local char-set string from/to UTF8 codings. */
char *localChar2Utf8Char(const char *localStr);
char *utf8AsnOcts2LocalStr(const AsnOcts &asnOcts);
// default local char-set is UTF8, so there is no conversion.
// if other local char-set is used, this virtual function must be
// overloaded.

#endif
