#include <stdio.h>
#include <string.h>
#include "realm_htmlindex.h"

/* class HtmlindexRealmLog */

int HtmlindexRealmLog::getFromMysqlResult(MYSQL_RES *result)
{
	serviceId = NULL;
	if(!result) return -1; /* open_mysql MUST precede! */

	MYSQL_ROW row = mysql_fetch_row(result);
	// Fields: serviceId(0), title(1), accessMethod(2),
	//	  author(3), keywords(4), modifiedTime(5)

	if(!row)
	{
		if(!mysql_eof(result))
		{
			fprintf(stderr, "Error: cannot fetch a row\n");
			return -1;
		}
		else return 0;
	}

	char modifiedTime[30];
	sprintf(modifiedTime, "%sZ", row[5]);
	reset( row[0], modifiedTime, (row[1]!=NULL), row[1],  row[2]);
		// serviceId, modifiedTime, valid, title, accessMethod

	return 1;
}

HtmlindexDifferenceResponse::HtmlindexDifferenceResponse(const char *lastTime, 
		const int ms)
{
	maxSize = ms;
	numRows = 0;
	if(lastTime) getMysqlResult(lastTime);
}

int HtmlindexDifferenceResponse::getMysqlResult(const char *lastTime)
{
	char queryStr[1000];
	MYSQL mysql;

	mysql_init(&mysql);
	
	// connect to mysql server
	if (!mysql_real_connect(&mysql, MYSQL_CLIENT_HOST, MYSQL_CLIENT_USER,
				MYSQL_CLIENT_PASSWD, MYSQL_CLIENT_DB, 0, NULL, 0))
	{
		fprintf(stderr, "Failed to connect to database: Error: %s\n",
				mysql_error(&mysql));
		return -1;
	}

	// get service index
	sprintf(queryStr, "SELECT * FROM " MYSQL_CLIENT_TABLE \
			" WHERE modifiedTime >= '%s' ORDER BY modifiedTime", lastTime);
	if(mysql_query(&mysql, queryStr))
	{
		fprintf(stderr, "Failed to query: Error: %s %s\n",
				mysql_error(&mysql), queryStr);
		mysql_close(&mysql);
		return -1;
	}
	mysqlResult = mysql_store_result(&mysql);
	if(!mysqlResult)
	{
		fprintf(stderr, "Failed to get result: Error: %s\n",
				mysql_error(&mysql));
		mysql_close(&mysql);
		return -1;
	}

	numRows = mysql_num_rows(mysqlResult);
	mysql_close(&mysql);
	return 0;
}

		
PisdMessage *HtmlindexDifferenceResponse::toPisdMessage()
{

	//if(getMysqlResult(mysqlResult, lastTime) < 0) return NULL;
	if(!mysqlResult) return NULL;

	int size;
	if(maxSize)
	{
		haveMore = numRows > maxSize;
		size = haveMore ? maxSize : numRows;
	}
	else /* infinite size allowed */
	{
		haveMore = false;
		size = numRows;
	}

	HtmlindexRealmLog *logArray = new HtmlindexRealmLog[size];
	for(int i=0; i<size; i++)
	{
		if(logArray[i].getFromMysqlResult(mysqlResult) == 0 )
		{
			fprintf(stderr, "Warning: no row!\n");
			size = i;
			break;
		}
	}

	numRows -= size;

	return makeDifferenceResponsePisdMessage(logArray, size, haveMore);
}
