#include <stdio.h>
#include <string.h>
#include "realm_serviceindex.h"
#include "server_mysql.h" // for defines

/* class ServiceindexRealmLog */

int ServiceindexRealmLog::getFromMysqlResult(MYSQL_RES *result)
{
	serviceId = NULL;
	if(!result) return -1; /* open_mysql MUST precede! */

	MYSQL_ROW row = mysql_fetch_row(result);
	// Fields: host(0), realm(1), serviceId(2), modifiedTime(3),
	// title(4), accessMethod(5)

	if(!row)
	{
		if(!mysql_eof(result))
		{
			fprintf(stderr, "Error: cannot fetch a row\n");
			return -1;
		}
		else return 0;
	}

	reset( row[2], row[3], (row[4]!=NULL), row[4],  row[5]);
		// serviceId, modifiedTime, valid, title, accessMethod

	return 1;
}

int ServiceindexDifferenceResponse::getMysqlResult(const char *realm,
		const char *host, const char *lastTime)
{
	char queryStr[1000];
	MYSQL mysql;

	mysql_init(&mysql);
	
	// connect to mysql server
	if (!mysql_real_connect(&mysql, MYSQL_SERVER_HOST, MYSQL_SERVER_USER,
				MYSQL_SERVER_PASSWD, MYSQL_SERVER_DB, 0, NULL, 0))
	{
		fprintf(stderr, "Failed to connect to database: Error: %s\n",
				mysql_error(&mysql));
		return -1;
	}

	// get service index
	sprintf(queryStr, "SELECT * FROM " MYSQL_SERVER_TABLE \
			" WHERE realm = '%s' AND host = '%s' AND " \
			"modifiedTime >= '%s' ORDER BY modifiedTime",
			realm, host, lastTime);
	if(mysql_query(&mysql, queryStr))
	{
		fprintf(stderr, "Failed to query: Error: %s %s\n",
				mysql_error(&mysql), queryStr);
		return -1;
	}
	mysqlResult = mysql_store_result(&mysql);
	if(!mysqlResult)
	{
		fprintf(stderr, "Failed to get result: Error: %s\n",
				mysql_error(&mysql));
		return -1;
	}

	numRows = mysql_num_rows(mysqlResult);
	mysql_close(&mysql);
	return 0;
}

		
PisdMessage *ServiceindexDifferenceResponse::toPisdMessage()
{

	//if(getMysqlResult(mysqlResult, lastTime) < 0) return NULL;

	int size;
	if(maxSize)
	{
		haveMore = numRows > maxSize;
		size = haveMore ? maxSize : numRows;
	}
	else /* infinite size allowed */
	{
		haveMore = false;
		size = numRows;
	}

	ServiceindexRealmLog *logArray = new ServiceindexRealmLog[size];
	for(int i=0; i<size; i++)
	{
		if(logArray[i].getFromMysqlResult(mysqlResult) == 0 )
		{
			fprintf(stderr, "Warning: no row!\n");
			size = i;
			break;
		}
	}

	numRows -= size;

	return makeDifferenceResponsePisdMessage(logArray, size, haveMore);
}
