/* Filename: s_daemon.C
 * PISDS daemon
 */

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "realm_htmlindex.h"
#include "pdu.h"
#include "pisd_net.h"
#include "server_mysql.h"

char *latestTimeOfServiceindex(const char *realm, const char *host);

int main(int argc, char **argv)
{
	if( argc != 1 )
	{
		cerr << "Usage: s_daemon\n";
		return 0;
	}

	// connect to PISDS
	int sock = startServer();
	if(sock < 0)
	{
		cerr << "Error: cannot start server.\n'";
		perror("");
		return 0;
	}

	while(1)
	{
		socklen_t clilen;
		struct sockaddr_in cliaddr;
		int connfd;
		PisdMessage *msg;

		clilen = sizeof(cliaddr);
		connfd = accept(sock, (struct sockaddr *)&cliaddr, &clilen);
		if(connfd < 0)
		{
			cerr << "Error: cannot accept client.\n";
			close(connfd);
			break;
		}

		// get update start
		char *pdu = readPdu(connfd);
		if(!pdu)
		{
			cerr << "Error: cannot receive update start.\n";
			close(connfd);
			break;
		}
		msg = decodePdu(pdu);
		delete pdu;
		if(!msg)
		{
			cerr << "Error: cannot decode pdu.\n";
			close(connfd);
			break;
		}
		if(msg->choiceId != PisdMessage::updateStartCid)
		{
			cerr << "Error: update start is not received.\n";
			close(connfd);
			break;
		}
		cout << "Received Update Start:\n" << *msg << endl;
		RealmUpdateStart rus(*msg->updateStart);
		cout << rus.realm << rus.host << rus.latestTime << endl;

		// send difference request
		int len;
		char *latestTime = latestTimeOfServiceindex(rus.realm, rus.host);
		if(!latestTime)
		{
			cerr << "No services\n";
			latestTime = "00000000000000Z";
		}
		RealmDifferenceRequest rdr(latestTime, rus.realm);
		msg = rdr.toPisdMessage();
		pdu = getPdu(*msg, len);
		delete msg;
		if(writen(connfd, pdu, len) < 0)
		{
			cerr << "Error: cannot send difference request.\n";
			close(connfd);
			break;
		}

		// MYSQL serviceindex
		ServerMysql sm(rus.host, rus.realm);
		sm.openMysql();
		bool haveMore;

		do
		{
			// get difference response
			pdu = readPdu(connfd);
			if(!pdu)
			{
				cerr << "Error: cannot receive update start.\n";
				close(connfd);
				break;
			}
			msg = decodePdu(pdu);
			//printPdu(pdu);
			delete pdu;
			if(!msg)
			{
				cerr << "Error: cannot decode pdu.\n";
				close(connfd);
				break;
			}
			if(msg->choiceId != PisdMessage::differenceResponseCid)
			{
				cerr << "Error: update start is not received.\n";
				close(connfd);
				break;
			}
			cout << "Received Differenc Respnonse:\n" << *msg << endl;
			sm.updateTable(msg->differenceResponse);

			// send status notify
			haveMore = (msg->differenceResponse->haveMore != NULL) ?
				(bool)*(msg->differenceResponse->haveMore) : false;
			if(haveMore)
			{
				// continue
				RealmStatusNotify rsn(statusOkContinue, "Update OK");
				PisdMessage *msg = rsn.toPisdMessage();
				pdu = getPdu(*msg, len);
				delete msg;
			}
			else
			{
				// close
				RealmStatusNotify rsn(statusOkClose, "Update OK (last)");
				PisdMessage *msg = rsn.toPisdMessage();
				pdu = getPdu(*msg, len);
				delete msg;
			}
			if(writen(connfd, pdu, len) < 0)
			{
				cerr << "Error: cannot send status notify.\n";
				close(connfd);
				break;
			}
		} while(haveMore);
		close(connfd);
		break;
	}

	close(sock);
	return 0;
}

/* Gets the latest modifiedTime of <realm/host> from 'serviceindex' table */
char *latestTimeOfServiceindex(const char *realm, const char *host)
{
	static char modifiedTime[30];
	MYSQL mysql;

	mysql_init(&mysql);
	if (!mysql_real_connect(&mysql, MYSQL_SERVER_HOST, MYSQL_SERVER_USER,
				MYSQL_SERVER_PASSWD, MYSQL_SERVER_DB, 0, NULL, 0))
	{
		cerr<< "Failed to connect to database: Error: "
			<< mysql_error(&mysql)
			<< endl;
		return NULL;
	}

	char queryStr[200];
	sprintf(queryStr, "SELECT modifiedTime FROM " MYSQL_SERVER_TABLE \
		" WHERE realm = '%s' AND host = '%s' ORDER BY modifiedTime DESC"
		" LIMIT 1", realm, host);
	if(mysql_query(&mysql, queryStr))
	{
		cerr<< "Failed to query: Error: "
			<< mysql_error(&mysql)
			<< " - " 
			<< queryStr
			<< endl;
		mysql_close(&mysql);
		return NULL;
	}
	MYSQL_RES *mysqlResult= mysql_store_result(&mysql);
	if(!mysqlResult)
	{
		cerr<< "Failed to get result: Error: "
			<< mysql_error(&mysql)
			<< endl;
		mysql_close(&mysql);
		return NULL;
	}
	MYSQL_ROW row = mysql_fetch_row(mysqlResult);
	if(!row)
	{
		cerr<< "Error: cannot fetch a row\n";
		mysql_close(&mysql);
		return NULL;
	}

	strcpy(modifiedTime, row[0]);
	//strcat(modifiedTime, "Z");
	mysql_close(&mysql);

	return modifiedTime;
}
